unit d2i_io;

{
disk2img BIOS Int 13h IO routines.
All functions that involve int 13h preserve and restore BP and DS, since
some clones trash these in the event of disk or drive errors.
}

{{$DEFINE NODISKIO} {for debugging; skips actually performing disk I/O}

interface

Function GetDriveStatus(drive:byte):byte;
Function ReadSectors(drive, head, track, sector, numtoread : Byte; buff: Pointer): word;
Function ResetDrive(drive:byte): Byte;
Function WriteSector(drive, head, track, sector : Byte; buff: Pointer): word;
Procedure GetMemDMAPageAligned(var DMAp:pointer;size:word);

var
  DriveMotorStatus:Byte Absolute $40:$3F;

implementation

uses
  support,
  d2i_globals;

Function ReadSectors(drive,head,track,sector,numtoread:byte; buff:pointer):word; assembler;
{drive = 0 for drive A:, 1 = B:, 80h = first hard drive}
asm
        push    ds
        push    bp
        mov     ah,02h
        mov     al,numtoread
        les     bx,buff                 {es:bx -> buffer}
        mov     ch,track
        mov     cl,sector
        mov     dh,head
        mov     dl,drive
        {$IFNDEF NODISKIO}
        int     13h
        {$ENDIF}
        {numsectors read in al, status in ah, cf=0 if successful}
        pop     bp
        pop     ds
end;

Function WriteSector(drive,head,track,sector:byte; buff:pointer): word; assembler;
{drive = 0 for drive A:, 1 = B:, 80h = first hard drive}
asm
        push    ds
        push    bp
        mov     ax,0301h                {read 1 sector}
        les     bx,buff                 {es:bx -> buffer}
        mov     ch,track
        mov     cl,sector
        mov     dh,head
        mov     dl,drive
        {$IFNDEF NODISKIO}
        int     13h
        {$ENDIF}
        mov     al,ah                   {status result in al}
        pop     bp
        pop     ds
end;

Function ResetDrive(drive:byte): Byte; assembler;
{drive = 0 for drive A:, 1 = B:, 80h = first hard drive}
asm
        push    ds
        push    bp
        xor     ax,ax
        mov     dl,drive
        {$IFNDEF NODISKIO}
        int     13h
        {$ENDIF}
        mov     al,ah                   {return status result in al}
        pop     bp
        pop     ds
end;

Function GetDriveStatus(drive:byte): Byte; assembler;
asm
        push    ds
        push    bp
        mov     ah,1
        {$IFNDEF NODISKIO}
        int     13h                     {return status result in al}
        {$ENDIF}
        pop     bp
        pop     ds
end;

Procedure getmemDMAPageAligned(var DMAp:pointer;size:word);
var
  basep,finalp:pointer;
  b:byte;
  w:word;
begin
  {allocate the buffer and see if it's within a DMA page boundary}
  getmem(basep,size);
  finalp:=l2p(p2l(basep)+size);
  asm
  mov al,byte ptr basep+3
  mov bl,byte ptr finalp+3
  mov cl,4
  shr al,cl
  shr bl,cl
  sub bl,al
  mov b,bl
  end;
  if b<>0 then begin
    {The buffer crosses a DMA page boundary, so we will unallocate our
    previous buffer, then recreate it ending at the page boundary, then
    create a new buffer right on the boundary.}
    asm
    and word ptr finalp+2,0F000h
    mov word ptr finalp,  00000h {finalp is now our ideal}
    end;
    w:=p2l(finalp)-p2l(basep); {difference between existing and ideal}
    freemem(basep,size); {basep:=nil; finalp:=nil;}
    getmem(basep,w); {allocate new spacer buffer ending at page boundary}
    getmem(finalp,size); {allocate final buffer starting at page boundary}
    freemem(basep,w); {return spacer buffer to heap}
    DMAp:=finalp;
  end else begin
    DMAp:=basep;
  end;
end;

end.
